@@ -56,7 +56,8 @@ def _do_deploy(api, args, formatter):
     if args.user_name:
         config.add_user(args.user_name, sudo=args.passwordless_sudo)
 
-    node = api.reserve_node(args.resource_class, capabilities=capabilities)
+    node = api.reserve_node(args.resource_class, capabilities=capabilities,
+                            candidates=args.candidate)
     instance = api.provision_node(node,
                                   image=args.image,
                                   nics=args.nics,
@@ -78,6 +79,12 @@ def _do_show(api, args, formatter):
     formatter.show(instances)
 
 
+def _do_wait(api, args, formatter):
+    instances = api.wait_for_provisioning(args.instance,
+                                          timeout=args.timeout)
+    formatter.show(instances)
+
+
 def _parse_args(args, config):
     parser = argparse.ArgumentParser(
         description='Deployment and Scheduling tool for Bare Metal')
@@ -102,12 +109,12 @@ def _parse_args(args, config):
 
     deploy = subparsers.add_parser('deploy')
     deploy.set_defaults(func=_do_deploy)
-    wait = deploy.add_mutually_exclusive_group()
-    wait.add_argument('--wait', type=int, default=1800,
-                      help='time (in seconds) to wait for node to become '
-                      'active')
-    wait.add_argument('--no-wait', action='store_true',
-                      help='disable waiting for deploy to finish')
+    wait_grp = deploy.add_mutually_exclusive_group()
+    wait_grp.add_argument('--wait', type=int, default=1800,
+                          help='time (in seconds) to wait for node to become '
+                          'active')
+    wait_grp.add_argument('--no-wait', action='store_true',
+                          help='disable waiting for deploy to finish')
     deploy.add_argument('--image', help='image to use (name or UUID)',
                         required=True)
     deploy.add_argument('--network', help='network to use (name or UUID)',
@@ -124,8 +131,11 @@ def _parse_args(args, config):
     deploy.add_argument('--ssh-public-key', help='SSH public key to load')
     deploy.add_argument('--hostname', help='Host name to use, defaults to '
                         'Node\'s name or UUID')
-    deploy.add_argument('--resource-class', required=True,
+    deploy.add_argument('--resource-class',
                         help='node resource class to deploy')
+    deploy.add_argument('--candidate', action='append',
+                        help='A candidate node to use for scheduling (can be '
+                        'specified several times)')
     deploy.add_argument('--user-name', help='Name of the admin user to create')
     deploy.add_argument('--passwordless-sudo', action='store_true',
                         help='allow password-less sudo for the user')
@@ -141,6 +151,12 @@ def _parse_args(args, config):
     show.set_defaults(func=_do_show)
     show.add_argument('instance', nargs='+', help='instance UUID(s)')
 
+    wait = subparsers.add_parser('wait')
+    wait.set_defaults(func=_do_wait)
+    wait.add_argument('instance', nargs='+', help='instance UUID(s)')
+    wait.add_argument('--timeout', type=int,
+                      help='time (in seconds) to wait for provisioning.')
+
     return parser.parse_args(args)
 
 
