@@ -33,6 +33,7 @@ class Base(testtools.TestCase):
         self.node = mock.Mock(spec=_os_api.NODE_FIELDS + ['to_dict'],
                               uuid='000', instance_uuid=None,
                               properties={'local_gb': 100},
+                              instance_info={},
                               maintenance=False, extra={})
         self.node.name = 'control-0'
 
@@ -72,6 +73,20 @@ class TestReserveNode(Base):
         node = self.pr.reserve_node('control')
 
         self.assertIn(node, nodes)
+        self.assertFalse(self.api.update_node.called)
+
+    def test_any_resource_class(self):
+        nodes = [
+            mock.Mock(spec=['uuid', 'name', 'properties'],
+                      properties={'local_gb': 100})
+        ]
+        self.api.list_nodes.return_value = nodes
+        self.api.reserve_node.side_effect = lambda n, instance_uuid: n
+
+        node = self.pr.reserve_node()
+
+        self.assertIn(node, nodes)
+        self.assertFalse(self.api.update_node.called)
 
     def test_with_capabilities(self):
         nodes = [
@@ -86,6 +101,56 @@ class TestReserveNode(Base):
         node = self.pr.reserve_node('control', {'answer': '42'})
 
         self.assertIs(node, expected)
+        self.api.update_node.assert_called_once_with(
+            node, {'/instance_info/capabilities': {'answer': '42'}})
+
+    def test_provided_node(self):
+        nodes = [
+            mock.Mock(spec=['uuid', 'name', 'properties'],
+                      properties={'local_gb': 100})
+        ]
+        self.api.reserve_node.side_effect = lambda n, instance_uuid: n
+
+        node = self.pr.reserve_node(candidates=nodes)
+
+        self.assertEqual(node, nodes[0])
+        self.assertFalse(self.api.list_nodes.called)
+        self.assertFalse(self.api.update_node.called)
+
+    def test_provided_nodes(self):
+        nodes = [
+            mock.Mock(spec=['uuid', 'name', 'properties'],
+                      properties={'local_gb': 100}),
+            mock.Mock(spec=['uuid', 'name', 'properties'],
+                      properties={'local_gb': 100})
+        ]
+        self.api.reserve_node.side_effect = lambda n, instance_uuid: n
+
+        node = self.pr.reserve_node(candidates=nodes)
+
+        self.assertEqual(node, nodes[0])
+        self.assertFalse(self.api.list_nodes.called)
+        self.assertFalse(self.api.update_node.called)
+
+    def test_nodes_filtered(self):
+        nodes = [
+            mock.Mock(spec=['uuid', 'name', 'properties', 'resource_class'],
+                      properties={'local_gb': 100}, resource_class='banana'),
+            mock.Mock(spec=['uuid', 'name', 'properties', 'resource_class'],
+                      properties={'local_gb': 100}, resource_class='compute'),
+            mock.Mock(spec=['uuid', 'name', 'properties', 'resource_class'],
+                      properties={'local_gb': 100, 'capabilities': 'cat:meow'},
+                      resource_class='compute'),
+        ]
+        self.api.reserve_node.side_effect = lambda n, instance_uuid: n
+
+        node = self.pr.reserve_node('compute', candidates=nodes,
+                                    capabilities={'cat': 'meow'})
+
+        self.assertEqual(node, nodes[2])
+        self.assertFalse(self.api.list_nodes.called)
+        self.api.update_node.assert_called_once_with(
+            node, {'/instance_info/capabilities': {'cat': 'meow'}})
 
 
 CLEAN_UP = {
@@ -119,6 +184,7 @@ class TestProvisionNode(Base):
             fixtures.MockPatchObject(_provisioner.Provisioner,
                                      '_wait_for_state', autospec=True))
         self.wait_mock = self.wait_fixture.mock
+        self.wait_mock.side_effect = lambda self, nodes, *a, **kw: nodes
 
     def test_ok(self):
         inst = self.pr.provision_node(self.node, 'image',
@@ -294,6 +360,77 @@ class TestProvisionNode(Base):
         self.assertFalse(self.api.release_node.called)
         self.assertFalse(self.api.delete_port.called)
 
+    def test_with_capabilities(self):
+        inst = self.pr.provision_node(self.node, 'image',
+                                      [{'network': 'network'}],
+                                      capabilities={'answer': '42'})
+        self.updates['/instance_info/capabilities'] = {'boot_option': 'local',
+                                                       'answer': '42'}
+
+        self.assertEqual(inst.uuid, self.node.uuid)
+        self.assertEqual(inst.node, self.node)
+
+        self.api.create_port.assert_called_once_with(
+            network_id=self.api.get_network.return_value.id)
+        self.api.attach_port_to_node.assert_called_once_with(
+            self.node.uuid, self.api.create_port.return_value.id)
+        self.api.update_node.assert_called_once_with(self.node, self.updates)
+        self.api.validate_node.assert_called_once_with(self.node,
+                                                       validate_deploy=True)
+        self.api.node_action.assert_called_once_with(self.node, 'active',
+                                                     configdrive=mock.ANY)
+        self.assertFalse(self.wait_mock.called)
+        self.assertFalse(self.api.release_node.called)
+        self.assertFalse(self.api.delete_port.called)
+
+    def test_with_existing_capabilities(self):
+        self.node.instance_info['capabilities'] = {'answer': '42'}
+        inst = self.pr.provision_node(self.node, 'image',
+                                      [{'network': 'network'}])
+        self.updates['/instance_info/capabilities'] = {'boot_option': 'local',
+                                                       'answer': '42'}
+
+        self.assertEqual(inst.uuid, self.node.uuid)
+        self.assertEqual(inst.node, self.node)
+
+        self.api.create_port.assert_called_once_with(
+            network_id=self.api.get_network.return_value.id)
+        self.api.attach_port_to_node.assert_called_once_with(
+            self.node.uuid, self.api.create_port.return_value.id)
+        self.api.update_node.assert_called_once_with(self.node, self.updates)
+        self.api.validate_node.assert_called_once_with(self.node,
+                                                       validate_deploy=True)
+        self.api.node_action.assert_called_once_with(self.node, 'active',
+                                                     configdrive=mock.ANY)
+        self.assertFalse(self.wait_mock.called)
+        self.assertFalse(self.api.release_node.called)
+        self.assertFalse(self.api.delete_port.called)
+
+    def test_override_existing_capabilities(self):
+        self.node.instance_info['capabilities'] = {'answer': '1',
+                                                   'cat': 'meow'}
+        inst = self.pr.provision_node(self.node, 'image',
+                                      [{'network': 'network'}],
+                                      capabilities={'answer': '42'})
+        self.updates['/instance_info/capabilities'] = {'boot_option': 'local',
+                                                       'answer': '42'}
+
+        self.assertEqual(inst.uuid, self.node.uuid)
+        self.assertEqual(inst.node, self.node)
+
+        self.api.create_port.assert_called_once_with(
+            network_id=self.api.get_network.return_value.id)
+        self.api.attach_port_to_node.assert_called_once_with(
+            self.node.uuid, self.api.create_port.return_value.id)
+        self.api.update_node.assert_called_once_with(self.node, self.updates)
+        self.api.validate_node.assert_called_once_with(self.node,
+                                                       validate_deploy=True)
+        self.api.node_action.assert_called_once_with(self.node, 'active',
+                                                     configdrive=mock.ANY)
+        self.assertFalse(self.wait_mock.called)
+        self.assertFalse(self.api.release_node.called)
+        self.assertFalse(self.api.delete_port.called)
+
     def test_with_wait(self):
         self.api.get_port.return_value = mock.Mock(
             spec=['fixed_ips'],
@@ -696,7 +833,8 @@ class TestWaitForState(Base):
         self.api.get_node.side_effect = nodes
 
         result = self.pr.wait_for_provisioning(['uuid1'])
-        self.assertEqual(nodes[-1:], result)
+        self.assertEqual(nodes[-1:], [inst.node for inst in result])
+        self.assertIsInstance(result[0], _instance.Instance)
 
         mock_sleep.assert_called_with(15)
         self.assertEqual(3, mock_sleep.call_count)
@@ -711,7 +849,9 @@ class TestWaitForState(Base):
         self.api.get_node.side_effect = nodes
 
         result = self.pr.wait_for_provisioning(['uuid1', 'uuid2'])
-        self.assertEqual(nodes[-2:], result)
+        self.assertEqual(nodes[-2:], [inst.node for inst in result])
+        for inst in result:
+            self.assertIsInstance(inst, _instance.Instance)
 
         mock_sleep.assert_called_with(15)
         self.assertEqual(2, mock_sleep.call_count)
@@ -758,7 +898,8 @@ class TestWaitForState(Base):
         self.api.get_node.side_effect = nodes
 
         result = self.pr.wait_for_provisioning(['uuid1'], delay=1)
-        self.assertEqual(nodes[-1:], result)
+        self.assertEqual(nodes[-1:], [inst.node for inst in result])
+        self.assertIsInstance(result[0], _instance.Instance)
 
         mock_sleep.assert_called_with(1)
         self.assertEqual(3, mock_sleep.call_count)
